# -*- coding: utf-8 -*-
#       icd_cam_open.py 2024.3.11   kita

#!! カム関係起動メニュー

# 関連ファイル
#   universal_cam.py  icd_cam2spline.py  icd_cam2spline.txt

#   履歴
# 2022.4.8  Point To Pointでカム曲線を作図するのを追加
# 2024.3.4  icd_cylinder_2D3D.py 追加

import tkinter as tk
from tkinter import *
from tkinter import ttk
import subprocess
import os
import universal_cam as uvc
import numpy as np
import math

import clr
clr.AddReference('sxnet')
from sxnet import *
SxSys.init(3999)

my_fn= os.path.basename(__file__)       # icd_*.py

fn = 'editor.ini'
if os.path.exists(fn):      # fnが存在するなら
    fp = open(fn, 'r')
    for line in fp.readlines():
        if line.find('#') > -1:  # #が含まれてるなら
            continue
        line = line.rstrip()    # 行末の改行を削除
        tmp = line.split(',')
        if tmp[0] == 'editor':
            editor = tmp[1]
else:
    editor = 'notepad'



#--------------------------------------------------------------------------------
# カムモデリング
#--------------------------------------------------------------------------------
def push_cam2spline():
    #ss_lmt = e_lmt.get()
    #lmt = float(ss_lmt)
    #cmd = 'py icd_cam2spline.py %f' % (lmt)
    cmd = 'py icd_cam2spline.py'
    esult = subprocess.Popen(cmd,shell=True)
    #print('\n    外部パーツのコピーを終了しました\n\n')

def push_cam2spline_help():
    #cmd = 'notepad '+path+'icd_dim_edit.hlp'
    #cmd = 'notepad icd_cam2spline.txt'
    cmd = editor +' icd_cam2spline.txt'
    esult = subprocess.Popen(cmd,shell=True)

#--------------------------------------------------------------------------------
# グリッパ開閉カム
#--------------------------------------------------------------------------------
def push_grp():
    cmd = 'py icd_grp.py'
    result = subprocess.Popen(cmd,shell=True)

"""
#--------------------------------------------------------------------------------
# 円筒板カム
#--------------------------------------------------------------------------------
def push_sld():
    cmd = 'py icd_cyl_cam.py'
    result = subprocess.Popen(cmd,shell=True)
"""

#--------------------------------------------------------------------------------
# icd_cylinder_2D3D.py 起動
#--------------------------------------------------------------------------------
def push_cylinder_2D3D():
    cmd = 'py icd_cylinder_2D3D.py'
    result = subprocess.Popen(cmd,shell=True)

#--------------------------------------------------------------------------------
#   Point To Point Cam 作図
#--------------------------------------------------------------------------------
def push_p2pcam():
    cam_nm = v_cam.get()
    if cam_nm.find('変形正弦') > -1:
        cam_no = 26
    elif  cam_nm.find('変形台形') > -1:
        cam_no = 25
    elif  cam_nm.find('変形等速度') > -1:
        cam_no = 27
    elif  cam_nm.find('Cycroidal') > -1:
        cam_no = 22
    #elif  cam_nm.find('等加速度') > -1:
    #    cam_no = 22

    cam = uvc.Universal_cam(cam_no)
    
    print('\n   始点をピックしてください')
    tmp = SxSys.getPos()
    p0 = np.array([tmp.x, tmp.y])  # 始点

    print('\n   終点をピックしてください')
    tmp = SxSys.getPos()
    p1 = np.array([tmp.x, tmp.y])  # 終点

    h = p1[1] - p0[1]   # ストローク(y座標)
    t_h = p1[0] - p0[0]  # ストローク時間(x座標)

    nn = 100    #分割数

    sva_all = cam.sva_nn(nn, 0)
    Vm = max(abs(max(sva_all[1])),abs(min(sva_all[1])))
    #max_A = max(abs(max(sva_all[2])),abs(min(sva_all[2])))
    Am = cam.a_mp
    #print(Vm, Am)
    # Scale
    scale = math.ceil(h/2.0/Am) #切り上げ
    inf_sys = SxSys.getInf()
    scale_sys = inf_sys.scale       # スケール 尺度 Scale

    #SxPos List
    sx_pnt_list0 = []   # 変位   
    sx_pnt_list1 = []   # 速度
    sx_pnt_list2 = []   # 加速度
    
    for i in range(nn+1):
        tmp00 = np.array([t_h*i/nn, h*sva_all[0][i]])
        #tmp01 = np.array([t_h*i/nn, h*sva_all[1][i]/t_h])
        #tmp02 = np.array([t_h*i/nn, h*sva_all[2][i]/t_h**2])
        #tmp01 = np.array([t_h*i/nn, h*sva_all[1][i]])
        #tmp02 = np.array([t_h*i/nn, h*sva_all[2][i]])
        tmp01 = np.array([t_h*i/nn, sva_all[1][i]*scale])
        tmp02 = np.array([t_h*i/nn, sva_all[2][i]*scale])
        tmp0 = p0 + tmp00   # 変位
        tmp1 = p0 + tmp01   # 速度
        tmp2 = p0 + tmp02   # 加速度
        tmp0_pos = SxPos(tmp0[0], tmp0[1])
        tmp1_pos = SxPos(tmp1[0], tmp1[1])
        tmp2_pos = SxPos(tmp2[0], tmp2[1])
        sx_pnt_list0.append(tmp0_pos)      
        sx_pnt_list1.append(tmp1_pos)      
        sx_pnt_list2.append(tmp2_pos)      
    
    pos0 = SxPos(p0[0], p0[1])
    pos_txt = SxPos(p0[0], p0[1]+50.0/scale_sys)
     
    ss = 'Vm = %1.3f  Am = %1.3f' % (Vm, Am)
    text = [cam_nm, ss]
    seg_txt = SxEntSeg.createText(pos_txt,text, True, SxEntSeg.DIM_WORD, SxEntSeg.TEXT_BASE_LB, 0.0) 
    
    # 変位　実線、水色
    #SxSys.setPenStyle(SxSys.STYLE_SOLID)    # 実線
    #SxSys.setPenWidth(SxSys.WIDTH_MIDDLE)    # 中線
    seg = SxEntSeg.createSpline2D(sx_pnt_list0,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0)
    SxEnt.setColor(seg, SxSys.COLOR_GREEN)   # 緑色
    SxEnt.setLineStyle(seg, SxSys.STYLE_SOLID)  #実線
    SxEnt.setLineWidth(seg, SxSys.WIDTH_MIDDLE)  # 中線
 
    # 速度　点線、空色
    seg = SxEntSeg.createSpline2D(sx_pnt_list1,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0)
    SxEnt.setColor(seg, SxSys.COLOR_SKY_BLUE)   # 空色
    SxEnt.setLineStyle(seg, SxSys.STYLE_DOT)  #点線
    SxEnt.setLineWidth(seg, SxSys.WIDTH_THIN)  # 細線
 
 
    # 加速度　一点鎖線、赤色
    seg = SxEntSeg.createSpline2D(sx_pnt_list2,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0)
    SxEnt.setColor(seg, SxSys.COLOR_RED)   # 赤色
    SxEnt.setLineStyle(seg, SxSys.STYLE_ONE_DOT_CHAIN)  #一点鎖線
    SxEnt.setLineWidth(seg, SxSys.WIDTH_THIN)  # 細線
 


    print('\n   作図を終了しました\007\n')
    

#************************************
# 画面設定、ボタン等の部品配置
#************************************

dx = 60;  dy = 30; nx = 2.8; ny = 4
x00 = 5; x01 =x00+dx; y00 = 5
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title(my_fn)  #タイトル
#root.title('カム Ver.0.81')   #タイトル
s_size = '%dx%d+%d+%d' % (x00+dx*nx, y00+dy*ny, 1100, 0)
root.geometry(s_size)
#root.attributes("-toolwindow",1)    # 最小、最大化ボタン非表示
# blue  red  khaki3  cyan  white

btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'   #ラベル文字色
e_bcol = 'khaki4'


#----------------------------------------------------------------
# icd_cam2spline ボタン 
ss = 'カムモデリング補助'; num = 0
#----------------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_cam2spline, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00,y=y00+num*dy)

ss = 'ヘルプ';
btn = tk.Button(root,text = ss,command = push_cam2spline_help,bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+dx*2,y=y00+num*dy)
"""
ss = '  制 限 値  '; num = 1   # ラベル

label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+25,y=y00+(num)*dy-2)    # Labelの位置
e_lmt = tk.Entry(root,width=5,bg=e_bcol)  # エントリー部品の生成
e_lmt.place(x=x00+dx*1.6,y=y00+num*dy-2)
e_lmt.insert(tk.END,'0.5')
"""


#----------------------------------------------------------------
ss = 'グリッパー開閉円盤板カム'; num = 1
#----------------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_grp, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00,y=y00+num*dy)

"""
#----------------------------------------------------------------
ss = '昇降Cam展開図、Modeling'; num = 2
#----------------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_up_down, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00,y=y00+num*dy)
"""

#----------------------------------------------------------------
ss = 'p2pカム作図'; num = 2
#----------------------------------------------------------------
v_cam = tk.StringVar()
cb_cam = ttk.Combobox(root, width=10,textvariable = v_cam) #, style="BW.TLabel")
#cb_cam.bind('<<ComboboxSelected>>', cb_cam_selected)   # Event処理
cb_cam['values']=('変形正弦','変形台形','変形等速度','Cycroidal') #,'等加速度')
cb_cam.set('変形正弦')
cb_cam.place(x=x00,y=y00+num*dy)

btn = tk.Button(root,text = ss,command = push_p2pcam, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+85,y=y00+num*dy)

#----------------------------------------------------------------
ss = 'Cylinder_2D3D'; num= 3
#----------------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_cylinder_2D3D, bg='orange',fg='black') #ボタンの作成
btn.place(x=x00,y=y00+num*dy)


root.mainloop()
